function [A,b,Mass]=assemblyDom(prob)

	// Assemblage des matrices lmentaires
	// correspondant aux intgrales dans l'intrieur du
	// domaine prob.dom (pour les frontires voir assemblyBdy).
	
	xi_Gauss2D=[.5 .5 0];
	eta_Gauss2D=[0 .5 .5];
	w_Gauss2D=[1/6 1/6 1/6];

	nElem=prob.domain.elements;
	nNodes=prob.domain.nodes;

	A=spzeros(nNodes,nNodes);
	Mass=spzeros(nNodes,nNodes);
	b=zeros(nNodes,1);

	phi=shapeTri(xi_Gauss2D,eta_Gauss2D);
	gTilde=gradShapeTri();

	for k=1:nElem
		nodesTri=prob.domain.tri(:,k);
		x=prob.domain.x(nodesTri);
		y=prob.domain.y(nodesTri);
		[Ae,be,Me]=elemTri(x,y,prob.diffusion,prob.proportional,prob.source,prob.transport);
		A(nodesTri,nodesTri)=A(nodesTri,nodesTri)+Ae;
		Mass(nodesTri,nodesTri)=Mass(nodesTri,nodesTri)+Me;
		b(nodesTri)=b(nodesTri)+be;
	end

endfunction

function [A,b]=assemblyBdy(prob)

	// Assemblage des conditions aux limites sur
	// la frontire du domaine dcrit dans prob.dom

	u_Gauss1D=[-sqrt(3)/3 sqrt(3)/3];
	w_Gauss1D=[1 1];  

	BIG=1e100;

	nElem=prob.domain.elements;
	nNodes=prob.domain.nodes;

	A=spzeros(nNodes,nNodes);
	b=zeros(nNodes,1);

	for i=1:length(prob.bdycond)
		select prob.bdycond(i).type
		case 'robin'
			[Ar,br]=assembRobinCnd(prob.domain.sides(i),...
									prob.domain.x,...
									prob.domain.y,...
									prob.bdycond(i).sigma,...
									prob.bdycond(i).xi);
			A=A+Ar;b=b+br;
		case 'dirichlet'
			[Ad,bd]=assembRobinCnd(prob.domain.sides(i),...
									prob.domain.x,...
									prob.domain.y,...
									1,...
									prob.bdycond(i).g);
			A=A+BIG*Ad; b=b+BIG*bd;
		end
	end

endfunction

function [A,b]=assembRobinCnd(bdy,x,y,sigma,xi)

	// Matrice et vecteur pour une condition
	// aux limites de Robin, 
	//
	// p*du/dn + sigma*u = xi

	nNodes=length(x);

	A=spzeros(nNodes,nNodes);
	b=spzeros(nNodes,1); 

	phi=shapeTriBnd(u_Gauss1D);

	for i=1:size(bdy,2);

		nodes=bdy(:,i);
		xloc=x(nodes); yloc=y(nodes);
		xgauss=(xloc(1)+xloc(2))/2+(xloc(2)-xloc(1))/2*u_Gauss1D;
		ygauss=(yloc(1)+yloc(2))/2+(yloc(2)-yloc(1))/2*u_Gauss1D;
		Ls2=sqrt((xloc(2)-xloc(1))^2+(yloc(2)-yloc(1))^2)/2;
    
	    if typeof(sigma)=='function'
			sigmaTilde=sigma(xgauss,ygauss);
			Ae=phi*diag(w_Gauss1D.*sigmaTilde)*phi';
		else
			Ae=sigma*phi*diag(w_Gauss1D)*phi';
		end

		if typeof(xi)=='function'
			xiTilde=xi(xgauss,ygauss);
			be=phi*(w_Gauss1D.*xiTilde)';
		else
			be=xi*phi*(w_Gauss1D)';
		end

		A(nodes,nodes)=A(nodes,nodes)+Ae*Ls2;
		b(nodes)=b(nodes)+be*Ls2;

	end

endfunction

function [Ae,be,Me]=elemTri(x,y,p,c,f,v)

	// Matrices et vecteur lmentaires pour le problme
	//
	// -div(p*u)+c*u+v.grad(u)=f, 
	//
	// + condition de Neuman Homogne
	//
	// Ae : matrice de "raideur"
	// Me : matrice de "Masse"
	// be : vecteur lmentaire
	//
	// Interpolation linaire

	D=[x(2)-x(1) x(3)-x(1);y(2)-y(1) y(3)-y(1)];
	C=[x(1);y(1)];

	detD=det(D);

	ptGauss2D=D*[xi_Gauss2D;eta_Gauss2D];
	ptGauss2D(1,:)=ptGauss2D(1,:)+C(1);
	ptGauss2D(2,:)=ptGauss2D(2,:)+C(2);

//	for k=1:length(w_Gauss2D)
//		Ae=Ae+w_Gauss2D(k)*cTilde(k)*phi(:,k)*phi(:,k)';
//		Me=Me+w_Gauss2D(k)*phi(:,k)*phi(:,k)';
//		be=be+w_Gauss2D(k)*fTilde(k)*phi(:,k);
//	end

	// La mme chose que ci-dessus en plus rapide

	g=(D')\gTilde;

	if typeof(c)=='function'
		cTilde=c(ptGauss2D(1,:),ptGauss2D(2,:));
	else
		cTilde=c;
	end
	if typeof(f)=='function'
		fTilde=f(ptGauss2D(1,:),ptGauss2D(2,:));
	else
		fTilde=f;
	end

	Ae=phi*diag(w_Gauss2D.*cTilde)*phi'+phi*diag(w_Gauss2D)*[v v v]'*g;
	Me=phi*diag(w_Gauss2D)*phi';
	be=phi*(w_Gauss2D.*fTilde)';

    if typeof(p)=='function'
	  	pTilde=p(ptGauss2D(1,:),ptGauss2D(2,:));
    	Ae=Ae+w_Gauss2D*pTilde'*g'*g;
	else
		Ae=Ae+.5*g'*p*g;
	end

	Ae=Ae*detD; Me=Me*detD; be=be*detD;
 
endfunction

function [s]=shapeTri(xi,eta)

   // Valeurs des 3 fonctions de forme
   // aux points de coordonnees locales
   // xi,eta : 1 x n,  donc n couples (xi,eta).

   s=[1-xi-eta ; xi ; eta];

endfunction

function [ds]=gradShapeTri()

   // Gradients des fonctions de forme

   ds=[-1 -1;1 0;0 1]';

endfunction

function [s]=shapeTriBnd(u)

   // Valeurs des deux fonctions de forme sur
   // la frontire. Coordonnes normalises 
   // sur [-1 1];

   s=[.5-.5*u;.5+.5*u]

endfunction

function [grad]=getGrad(dom,u)

   // Calcul du gradient de la solution
   // a posteriori

   grad=zeros(2,dom.elements);

   for i=1:dom.elements
      nodes=dom.tri(:,i);
      x=dom.x(nodes);
      y=dom.y(nodes);
      D=[x(2)-x(1) x(3)-x(1);y(2)-y(1) y(3)-y(1)];
      grad(:,i)=D'\(gradShapeTri()*u(nodes));
   end
   
endfunction

function [x,y,tr]=getTrace(dom,u,ibnd)

   // rcupration de la trace de la solution
   // sur le bord ibnd. Les points sont classs
   // dans le sens triginomtrique.

	ind=dom.sides(ibnd);
	ind=[ind(1,:) ind(2,$)];
	tr=u(ind);
	x=dom.x(ind)';
	y=dom.y(ind)';  

endfunction

